<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:plm="http://www.plmxml.org/Schemas/PLMXMLSchema"
	xmlns:cim="http://www.coolemall.eu/DEBBComponent" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<!-- Specification of the output document -->		
	<xsl:output method="xml" version="1.0" encoding="UTF-8"
		indent="yes" />
	
	<xsl:param name="plmxmlFileName" select="plmxmlFileName" />	

	<xsl:template match="//*[ComponentId]" mode="component">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="translateComputingElement">
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="//*[ComponentId]"  mode="coolingDevice">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="translateCoolingDevice">
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="//*[ComponentId]"  mode="fan">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="translateFan">
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="//*[ComponentId]"  mode="heatsink">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="translateHeatsink">
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="translateComputingElement">
		<xsl:param name="instanceRefs"/>

		<xsl:call-template name="BasicDEBBPhysicalElementParameters" />
		
		<xsl:if test="Memory">
			<xsl:apply-templates select="Memory" />
		</xsl:if>
		
		<xsl:if test="Baseboard">
			<xsl:apply-templates select="Baseboard" />
		</xsl:if>
		
		<xsl:if test="Sensor">
			<xsl:apply-templates select="Sensor" />
		</xsl:if>
		
		<xsl:if test="PowerSupply">
			<xsl:apply-templates select="PowerSupply" />
		</xsl:if>
		
		<xsl:if test="Processor">
			<xsl:apply-templates select="Processor" />
		</xsl:if>

					 	
		<!--
		<xsl:if test="Heatsink">
			<xsl:apply-templates select="Heatsink" mode="device">
				<xsl:with-param name="instanceRefs" select="$instanceRefs" />
			</xsl:apply-templates>
		</xsl:if>	
		 -->		
		 
		 <!--
		<xsl:if test="Inlet">
			<xsl:apply-templates select="Inlet" mode="device">
				<xsl:with-param name="instanceRefs" select="$instanceRefs" />
			</xsl:apply-templates>
		</xsl:if>	
		
		<xsl:if test="Outlet">
			<xsl:apply-templates select="Outlet" mode="device">
				<xsl:with-param name="instanceRefs" select="$instanceRefs" />
			</xsl:apply-templates>
		</xsl:if>	
		 -->	
		<xsl:if test="Slot">
			<xsl:apply-templates select="Slot" />
		</xsl:if>	

		<xsl:if test="Size">
			<xsl:element name="parameter">
				<xsl:attribute name="name">size</xsl:attribute>	
					<xsl:element name="value">
					<xsl:value-of select="Size" />
				</xsl:element>
			</xsl:element>
		</xsl:if>	

		<xsl:if test="Network">
			<xsl:apply-templates select="Network" />
		</xsl:if>
		
	</xsl:template>

	<xsl:template match="Baseboard">
		<xsl:element name="parameter">
			<xsl:attribute name="name">baseboard</xsl:attribute>
			<!-- Don't call BasicDEBBPhysicalElementParameters, because parameter 
				cannot contain parameters. Must be parameter containing properties. -->
			<xsl:call-template name="BasicDEBBPhysicalElementProperties" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="Sensor">
		<xsl:element name="parameter">
			<xsl:attribute name="name">sensor</xsl:attribute>
			<xsl:element name="property">
				<xsl:attribute name="name">class</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Class" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">unit</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Unit" />
				</xsl:element>
			</xsl:element>
			<xsl:if test="MinValue">
				<xsl:element name="property">
					<xsl:attribute name="name">minValue</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="MinValue" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="MaxValue">
				<xsl:element name="property">
					<xsl:attribute name="name">maxValue</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="MaxValue" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="Factor">
				<xsl:element name="property">
					<xsl:attribute name="name">factor</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Factor" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="Accuracy">
				<xsl:element name="property">
					<xsl:attribute name="name">accuracy</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Accuracy" />
					</xsl:element>
				</xsl:element>
			</xsl:if>

			<!-- Don't call BasicDEBBPhysicalElementParameters, because parameter 
				cannot contain parameters. Must be parameter containing properties. -->
			<xsl:call-template name="BasicDEBBPhysicalElementProperties" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="Processor">
		<xsl:element name="computingResource">
			<xsl:attribute name="class">Processor</xsl:attribute>
			<xsl:call-template name="BasicDEBBPhysicalElementParameters" />
			<xsl:if test="Cores">
				<xsl:element name="computingResource">
					<xsl:attribute name="class">Core</xsl:attribute>
					<xsl:attribute name="count">
						<xsl:value-of select="Cores" />
					</xsl:attribute>
				</xsl:element>
			</xsl:if>
			<xsl:element name="parameter">
				<xsl:attribute name="name">speed</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxClockSpeed" />
				</xsl:element>
			</xsl:element>
			<xsl:if test="PState | CState">
				<!-- If PState or CState elements present add powerProfile element -->
				<xsl:element name="profile">
					<xsl:element name="powerProfile">
						
						<xsl:if test="CState">
							<!-- If any CState is present add powerStates element -->
							<xsl:element name="powerStates">
								<xsl:apply-templates select="CState" />
							</xsl:element>
						</xsl:if>
						
						<!--  
						<xsl:if test="PState/LoadPowerUsage">
							<xsl:apply-templates select="PState/LoadPowerUsage" />
						</xsl:if>
						-->
						<xsl:apply-templates select="PState" />						
					</xsl:element>
				</xsl:element>
			</xsl:if>
			
			<xsl:if test="TDP">
				<xsl:element name="parameter">
					<xsl:attribute name="name">tdp</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="TDP" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
		</xsl:element>
	</xsl:template>

	<xsl:template match="PState">
		<xsl:element name="parameter">
			<xsl:attribute name="name">pState</xsl:attribute>
			<xsl:element name="property">
				<xsl:attribute name="name">name</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="State" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">frequency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Frequency" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">voltage</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Voltage" />
				</xsl:element>
			</xsl:element>
			<!-- 	
			<xsl:element name="property">
				<xsl:attribute name="name">powerUsageMin</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="PowerUsageMin" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">powerUsageMax</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="PowerUsageMax" />
				</xsl:element>
			</xsl:element>
			-->	
			<xsl:apply-templates select="LoadPowerUsage" mode="parameter" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="CState">

		<xsl:element name="powerState">
			<xsl:element name="name">
				<xsl:value-of select="State" />
			</xsl:element>
			<xsl:element name="powerUsage">
				<xsl:value-of select="PowerUsage" />
			</xsl:element>			
		</xsl:element>
	</xsl:template>

	<xsl:template match="LoadPowerUsage" mode="parameter">
		<xsl:element name="property">
			<xsl:attribute name="name">loadPowerUsage</xsl:attribute>
			<xsl:element name="value">
				<xsl:value-of select="Load" />
			</xsl:element>
			<xsl:element name="value">
				<xsl:value-of select="PowerUsage" />
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="LoadPowerUsage" mode="element">
		<xsl:element name="loadPowerUsage">
			<xsl:element name="load">
				<xsl:value-of select="Load" />
			</xsl:element>
			<xsl:element name="powerUsage">
				<xsl:value-of select="PowerUsage" />
			</xsl:element>
		</xsl:element>
	</xsl:template>

	<xsl:template match="PowerSupply">
		<xsl:element name="parameter">
			<xsl:attribute name="name">powerSupply</xsl:attribute>
			
			<xsl:element name="property">
				<xsl:attribute name="name">class</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Class" />
				</xsl:element>
			</xsl:element>

			<xsl:element name="property">
				<xsl:attribute name="name">totalOutputPower</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="TotalOutputPower" />
				</xsl:element>
			</xsl:element>
		
			<xsl:element name="property">
				<xsl:attribute name="name">efficiency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Efficiency" />
				</xsl:element>
			</xsl:element>
			
			<!-- TODO: PowerProfile -->
			
			<!-- Don't call BasicDEBBPhysicalElementParameters, because parameter 
				cannot contain parameters. Must be parameter containing properties. -->
			<xsl:call-template name="BasicDEBBPhysicalElementProperties" />
		</xsl:element>
	</xsl:template>

	<xsl:template match="Memory">
		<xsl:element name="resourceUnit">
			<xsl:attribute name="class">Memory</xsl:attribute>
			<xsl:element name="amount">
				<xsl:value-of select="Capacity" />
			</xsl:element>
			<xsl:call-template name="BasicDEBBPhysicalElementParameters" />
		</xsl:element>
	</xsl:template>

	
	<xsl:template name="translateCoolingDevice">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="CoolingDeviceType" >
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>	
	
	<xsl:template name="CoolingDeviceType">
 		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="BasicDEBBPhysicalElementParameters" />
		
		<xsl:if test="Class">
			<xsl:element name="parameter">
				<xsl:attribute name="name">class</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Class" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="MaxCoolingCapacity">
			<xsl:element name="parameter">
				<xsl:attribute name="name">maxCoolingCapacity</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxCoolingCapacity" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="MaxAirThroughput">
			<xsl:element name="parameter">
				<xsl:attribute name="name">maxAirThroughput</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxAirThroughput" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="CRAHFanEfficiency">
			<xsl:element name="parameter">
				<xsl:attribute name="name">CRAHFanEfficiency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="CRAHFanEfficiency" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		 
		 <xsl:if test="CoolingCoilEfficiency">
			<xsl:element name="parameter">
				<xsl:attribute name="name">coolingCoilEfficiency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="CoolingCoilEfficiency" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		 <xsl:if test="DeltaThEx">
			<xsl:element name="parameter">
				<xsl:attribute name="name">deltaThEx</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="DeltaThEx" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="CWT">
			<xsl:element name="parameter">
				<xsl:attribute name="name">CWT</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="CWT" />
				</xsl:element>
			</xsl:element>
		</xsl:if>

		<xsl:if test="DeltaThDryCooler">
			<xsl:element name="parameter">
				<xsl:attribute name="name">deltaThDryCooler</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="DeltaThDryCooler" />
				</xsl:element>
			</xsl:element>
		</xsl:if>


		<xsl:if test="CoolingCapacityRated">
			<xsl:element name="parameter">
				<xsl:attribute name="name">coolingCapacityRated</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="CoolingCapacityRated" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="EERRated">
			<xsl:element name="parameter">
				<xsl:attribute name="name">EERRated</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="EERRated" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
  
		<xsl:apply-templates select="EnergyEfficiencyRatio/Item"/>
				
		<!-- Air throughput connected, advanced, technical data -->					
		<xsl:apply-templates select="AirThroughputProfile" />
		
		<xsl:variable name="tokenizedText" select="tokenize($instanceRefs, ' ')" />
		<xsl:variable name="outsideDoc" select="document($plmxmlFileName)" />
		<xsl:for-each select="$tokenizedText">
			<xsl:variable name="componentId" select="." />
			<xsl:apply-templates select="$outsideDoc/plm:PLMXML/plm:ProductDef/plm:InstanceGraph/plm:ProductInstance[@partRef=$componentId]" mode="updateCoolingDevice">
			</xsl:apply-templates>
		</xsl:for-each>	
		
	</xsl:template>

	<xsl:template match="Item">
		<xsl:element name="parameter">
			<xsl:attribute name="name">EnergyEfficiencyRatio</xsl:attribute>
			<xsl:element name="property">
				<xsl:attribute name="name">LWT</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="LWT" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">CWT</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="CWT" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">Capacity</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Capacity" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">PowerUsage</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="PowerUsage" />
				</xsl:element>
			</xsl:element>
			<xsl:element name="property">
				<xsl:attribute name="name">EER</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="EER" />
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	
	<xsl:template match="AirThroughputProfile">
		<xsl:element name="profile">
			<xsl:element name="airThroughputProfile">
				<xsl:element name="airThroughputStates">
					<xsl:apply-templates select="FlowState" mode="air"/>
				</xsl:element>
			</xsl:element>						
		</xsl:element>
	</xsl:template>
	
	<!--
	<xsl:template match="PowerUsageProfile" mode="air">
		<xsl:element name="profile">
				<xsl:element name="airThroughputProfile">
					<xsl:element name="airThroughputStates">
						<xsl:apply-templates select="FlowState" mode="air"/>
					</xsl:element>
				</xsl:element>						
			</xsl:element>
	</xsl:template>	
	-->
	
	<xsl:template match="PowerUsageProfile" mode="air">
		<xsl:element name="airThroughputStates">
			<xsl:apply-templates select="FlowState" mode="air"/>
		</xsl:element>
	</xsl:template>	
	
	<xsl:template match="PowerUsageProfile" mode="power">
		<xsl:element name="profile">
				<xsl:element name="powerProfile">
					<xsl:element name="powerStates">
						<xsl:apply-templates select="FlowState" mode="power"/>
					</xsl:element>
				</xsl:element>						
			</xsl:element>
	</xsl:template>	

	
	<xsl:template match="FlowState" mode="power">
		<xsl:element name="powerState">
			<xsl:call-template name="FlowStateItem"/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="FlowState" mode="air">
		<xsl:element name="airThroughputState">
			<xsl:call-template name="FlowStateItem"/>
		</xsl:element>
	</xsl:template>

	<xsl:template name ="FlowStateItem">
		<xsl:element name="name">
			<xsl:value-of select="State" />
		</xsl:element>
		<xsl:if test="Flow">
			<xsl:element name="value">
				<xsl:value-of select="Flow" />
			</xsl:element>
		</xsl:if>
		<xsl:if test="PowerUsage">
			<xsl:element name="powerUsage">
				<xsl:value-of select="PowerUsage" />
			</xsl:element>
		</xsl:if>
	</xsl:template>
	


	<xsl:template name="translateHeatsink">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="HeatsinkType" >
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>	
	
	<xsl:template name="HeatsinkType">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="BasicDEBBPhysicalElementParameters" />

		<xsl:if test="TransferRate">
			<xsl:element name="parameter">
				<xsl:attribute name="name">efficiency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Efficiency" />
				</xsl:element>
			</xsl:element>
		</xsl:if>	
		
		<xsl:variable name="tokenizedText" select="tokenize($instanceRefs, ' ')" />
		<xsl:variable name="outsideDoc" select="document($plmxmlFileName)" />
		<xsl:for-each select="$tokenizedText">
			<xsl:variable name="componentId" select="." />
			<xsl:apply-templates select="$outsideDoc/plm:PLMXML/plm:ProductDef/plm:InstanceGraph/plm:ProductInstance[@partRef=$componentId]" mode="updateHeatsinkDevice">
			</xsl:apply-templates>
		</xsl:for-each>		
		
	</xsl:template>

	
	<xsl:template name="FlowPumpType">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="BasicDEBBPhysicalElementParameters" />

		<xsl:if test="MaxRPM">
			<xsl:element name="parameter">
				<xsl:attribute name="name">maxRPM</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxRPM" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
			
		<xsl:if test="Efficiency">
			<xsl:element name="parameter">
				<xsl:attribute name="name">efficiency</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Efficiency" />
				</xsl:element>
			</xsl:element>
		</xsl:if>	

		<xsl:variable name="tokenizedText" select="tokenize($instanceRefs, ' ')" />
		<xsl:variable name="outsideDoc" select="document($plmxmlFileName)" />
		<xsl:for-each select="$tokenizedText">
			<xsl:variable name="componentId" select="." />
			<xsl:apply-templates select="$outsideDoc/plm:PLMXML/plm:ProductDef/plm:InstanceGraph/plm:ProductInstance[@partRef=$componentId]" mode="updateCoolingDevice">
			</xsl:apply-templates>
		</xsl:for-each>		
		
		<xsl:element name="profile">
			<xsl:element name="airThroughputProfile">
				<xsl:apply-templates select="PowerUsageProfile" mode="air"/>
			</xsl:element>					
		</xsl:element>
		
		<!--
		<xsl:element name="profile">
			<xsl:element name="airThroughputProfile">
				<xsl:element name="airThroughputStates">
					<xsl:apply-templates select="PowerUsageProfile/FlowState" mode="air"/>
				</xsl:element>
			</xsl:element>					
		</xsl:element>
		-->
		<!--		
		<xsl:apply-templates select="PowerUsageProfile" mode="air" />
		-->	
	</xsl:template>
	
	<!-- Template for transforming basic parameters of all devices of any type -->
	<xsl:template name="BasicDEBBPhysicalElementParameters">
		<xsl:if test="Label">
			<xsl:element name="parameter">
				<xsl:attribute name="name">label</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Label" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
	
		<xsl:if test="Manufacturer">
			<xsl:element name="parameter">
				<xsl:attribute name="name">manufacturer</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Manufacturer" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		<xsl:if test="Product">
			<xsl:element name="parameter">
				<xsl:attribute name="name">product</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Product" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		<xsl:if test="MaxPower">
			<xsl:element name="parameter">
				<xsl:attribute name="name">maxPower</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxPower" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="PowerUsage">
			<xsl:element name="parameter">
				<xsl:attribute name="name">powerUsage</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="PowerUsage" />
				</xsl:element>
			</xsl:element>
		</xsl:if>

		<!-- TODO: PowerUsageProfile -->

		<xsl:if test="Type">
			<xsl:element name="parameter">
				<xsl:attribute name="name">type</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Type" />
				</xsl:element>
			</xsl:element>
		</xsl:if>	
	</xsl:template>

	<xsl:template name="BasicDEBBPhysicalElementProperties">
		
		<xsl:if test="Label">
			<xsl:element name="property">
				<xsl:attribute name="name">label</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Label" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="Manufacturer">
			<xsl:element name="property">
				<xsl:attribute name="name">manufacturer</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Manufacturer" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="Product">
			<xsl:element name="property">
				<xsl:attribute name="name">product</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Product" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="MaxPower">
			<xsl:element name="property">
				<xsl:attribute name="name">maxPower</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="MaxPower" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
		
		<xsl:if test="PowerUsage">
			<xsl:element name="property">
				<xsl:attribute name="name">powerUsage</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="PowerUsage" />
				</xsl:element>
			</xsl:element>
		</xsl:if>
	
		<xsl:if test="Type">
			<xsl:element name="property">
				<xsl:attribute name="name">type</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Type" />
				</xsl:element>
			</xsl:element>
		</xsl:if>

	</xsl:template>


	<xsl:template name="translateFan">
		<xsl:param name="instanceRefs"/>
		<xsl:call-template name="FlowPumpType" >
			<xsl:with-param name="instanceRefs" select="$instanceRefs" />
		</xsl:call-template>
	</xsl:template>	
		
	<!--
	<xsl:template match="Heatsink" mode="device">
		<xsl:param name="instanceRefs"/>
		<xsl:element name="device">
			<xsl:attribute name="class">Heatsink</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="Label" />
			</xsl:attribute>

			<xsl:call-template name="HeatsinkType" >
				<xsl:with-param name="instanceRefs" select="$instanceRefs" />
			</xsl:call-template>
		</xsl:element>
		
	</xsl:template>	
	
		
	<xsl:template match="Inlet" mode="device">
		<xsl:param name="instanceRefs"/>
		<xsl:element name="device">
			<xsl:attribute name="class">Inlet</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="Label" />
			</xsl:attribute>

			<xsl:call-template name="FlowPumpType" >
				<xsl:with-param name="instanceRefs" select="$instanceRefs" />
			</xsl:call-template>
		</xsl:element>
	</xsl:template>	
	
	<xsl:template match="Outlet" mode="device">
		<xsl:param name="instanceRefs"/>
		<xsl:element name="device">
			<xsl:attribute name="class">Outlet</xsl:attribute>
			<xsl:attribute name="name">
				<xsl:value-of select="Label" />
			</xsl:attribute>

			<xsl:call-template name="FlowPumpType" >
				<xsl:with-param name="instanceRefs" select="$instanceRefs" />
			</xsl:call-template>
		</xsl:element>
	</xsl:template>	
		 -->	

	<xsl:template match="Slot">
		<xsl:element name="parameter">
			<xsl:attribute name="name">slot</xsl:attribute>	
			
			<xsl:call-template name="BasicDEBBConnectorTypeProperties"/>
		</xsl:element>	
	</xsl:template>
	
	<xsl:template name="BasicDEBBConnectorTypeProperties">
		<xsl:if test="Number">
			<xsl:element name="property">
				<xsl:attribute name="name">number</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="Number" />
				</xsl:element>
			</xsl:element>
		</xsl:if>

		<xsl:element name="property">
			<xsl:attribute name="name">connectorType</xsl:attribute>
			<xsl:element name="value">
				<xsl:value-of select="ConnectorType" />
			</xsl:element>
		</xsl:element>

		<xsl:element name="property">
			<xsl:attribute name="name">label</xsl:attribute>
			<xsl:element name="value">
				<xsl:value-of select="Label" />
			</xsl:element>
		</xsl:element>
		
		<xsl:if test="AvailSpace">
			<xsl:element name="property">
				<xsl:attribute name="name">availSpace</xsl:attribute>
				<xsl:element name="value">
					<xsl:value-of select="AvailSpace" />
				</xsl:element>
			</xsl:element>
		</xsl:if>	
	</xsl:template>
	
	<xsl:template match="Network">
		<xsl:element name="parameter">
			<xsl:attribute name="name">network</xsl:attribute>
			
			<xsl:if test="Interface">
				<xsl:element name="property">
					<xsl:attribute name="name">interface</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Interface" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="Technology">
				<xsl:element name="property">
					<xsl:attribute name="name">technology</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="Technology" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
			<xsl:if test="MaxBandwidth">
				<xsl:element name="property">
					<xsl:attribute name="name">maxBandwidth</xsl:attribute>
					<xsl:element name="value">
						<xsl:value-of select="MaxBandwidth" />
					</xsl:element>
				</xsl:element>
			</xsl:if>
	
			<!-- Don't call BasicDEBBPhysicalElementParameters, because parameter 
				cannot contain parameters. Must be parameter containing properties. -->
			<xsl:call-template name="BasicDEBBPhysicalElementProperties" />
		</xsl:element>
	</xsl:template>
</xsl:stylesheet>